@@ -171,18 +171,18 @@ int main()
         /**
          * Ensure response extensions are only received if sent
          *
-         *= https://tools.ietf.org/rfc/rfc8446#section-4.2
+         *= https://www.rfc-editor.org/rfc/rfc8446#section-4.2
          *= type=test
          *# Upon receiving such an extension, an endpoint MUST abort the handshake
          *# with an "unsupported_extension" alert.
          *
-         *= https://tools.ietf.org/rfc/rfc7627#section-5.3
+         *= https://www.rfc-editor.org/rfc/rfc7627#section-5.3
          *= type=test
          *# If the original session did not use the "extended_master_secret"
          *# extension but the new ServerHello contains the extension, the
          *# client MUST abort the handshake.
          *
-         *= https://tools.ietf.org/rfc/rfc8446#4.1.4
+         *= https://www.rfc-editor.org/rfc/rfc8446#4.1.4
          *= type=test
          *# As with the ServerHello, a HelloRetryRequest MUST NOT contain any
          *# extensions that were not first offered by the client in its
@@ -267,13 +267,13 @@ int main()
         /**
          * Ensure correct response extension send behavior
          *
-         *= https://tools.ietf.org/rfc/rfc8446#section-4.2
+         *= https://www.rfc-editor.org/rfc/rfc8446#section-4.2
          *= type=test
          *# Implementations MUST NOT send extension responses if the remote
          *# endpoint did not send the corresponding extension requests, with the
          *# exception of the "cookie" extension in the HelloRetryRequest.
          *
-         *= https://tools.ietf.org/rfc/rfc8446#4.1.4
+         *= https://www.rfc-editor.org/rfc/rfc8446#4.1.4
          *= type=test
          *# As with the ServerHello, a HelloRetryRequest MUST NOT contain any
          *# extensions that were not first offered by the client in its
