@@ -1478,6 +1478,232 @@ int main(int argc, char **argv)
         EXPECT_EQUAL(client->ticket_lifetime_hint, 0);
     }
 
+    /* Test: Server disables tls12 tickets */
+    {
+        DEFER_CLEANUP(struct s2n_config *forward_secret_config = s2n_config_new(),
+                s2n_config_ptr_free);
+        EXPECT_NOT_NULL(forward_secret_config);
+        EXPECT_SUCCESS(s2n_config_require_ticket_forward_secrecy(forward_secret_config, true));
+        EXPECT_SUCCESS(s2n_config_set_cipher_preferences(forward_secret_config, "default_tls13"));
+        EXPECT_SUCCESS(s2n_config_add_cert_chain_and_key_to_store(forward_secret_config,
+                chain_and_key));
+        EXPECT_SUCCESS(s2n_config_set_session_tickets_onoff(forward_secret_config, 1));
+        EXPECT_SUCCESS(s2n_config_add_ticket_crypto_key(forward_secret_config, ticket_key_name1,
+                s2n_array_len(ticket_key_name1), ticket_key1, s2n_array_len(ticket_key1), 0));
+
+        DEFER_CLEANUP(struct s2n_config *tls12_client_config = s2n_config_new(),
+                s2n_config_ptr_free);
+        EXPECT_NOT_NULL(tls12_client_config);
+        EXPECT_SUCCESS(s2n_config_set_session_tickets_onoff(tls12_client_config, 1));
+        EXPECT_SUCCESS(s2n_config_set_unsafe_for_testing(tls12_client_config));
+        /* Security policy that does not support TLS1.3 */
+        EXPECT_SUCCESS(s2n_config_set_cipher_preferences(tls12_client_config, "20240501"));
+
+        DEFER_CLEANUP(struct s2n_config *tls13_client_config = s2n_config_new(),
+                s2n_config_ptr_free);
+        EXPECT_NOT_NULL(tls12_client_config);
+        EXPECT_SUCCESS(s2n_config_set_session_tickets_onoff(tls13_client_config, 1));
+        EXPECT_SUCCESS(s2n_config_set_unsafe_for_testing(tls13_client_config));
+        /* Security policy that does support TLS1.3 */
+        EXPECT_SUCCESS(s2n_config_set_cipher_preferences(tls13_client_config, "default_tls13"));
+
+        /* Server does not send ticket when forward secrecy is enforced and TLS1.2 is negotiated */
+        {
+            DEFER_CLEANUP(struct s2n_connection *client = s2n_connection_new(S2N_CLIENT),
+                    s2n_connection_ptr_free);
+            EXPECT_NOT_NULL(client);
+            DEFER_CLEANUP(struct s2n_connection *server = s2n_connection_new(S2N_SERVER),
+                    s2n_connection_ptr_free);
+            EXPECT_NOT_NULL(server);
+
+            EXPECT_SUCCESS(s2n_connection_set_config(client, tls12_client_config));
+            EXPECT_SUCCESS(s2n_connection_set_config(server, forward_secret_config));
+
+            DEFER_CLEANUP(struct s2n_test_io_stuffer_pair test_io = { 0 },
+                    s2n_io_stuffer_pair_free);
+            EXPECT_OK(s2n_io_stuffer_pair_init(&test_io));
+            EXPECT_OK(s2n_connections_set_io_stuffer_pair(client, server, &test_io));
+
+            EXPECT_SUCCESS(s2n_negotiate_test_server_and_client(server, client));
+            EXPECT_EQUAL(client->actual_protocol_version, S2N_TLS12);
+            EXPECT_EQUAL(server->actual_protocol_version, S2N_TLS12);
+
+            uint16_t tickets_sent = 0;
+            EXPECT_SUCCESS(s2n_connection_get_tickets_sent(server, &tickets_sent));
+            EXPECT_EQUAL(tickets_sent, 0);
+        }
+
+        /* Server does send tickets when forward secrecy is enforced and TLS1.3 is negotiated */
+        if (s2n_is_tls13_fully_supported()) {
+            DEFER_CLEANUP(struct s2n_connection *client = s2n_connection_new(S2N_CLIENT),
+                    s2n_connection_ptr_free);
+            EXPECT_NOT_NULL(client);
+            DEFER_CLEANUP(struct s2n_connection *server = s2n_connection_new(S2N_SERVER),
+                    s2n_connection_ptr_free);
+            EXPECT_NOT_NULL(server);
+
+            EXPECT_SUCCESS(s2n_connection_set_config(client, tls13_client_config));
+            EXPECT_SUCCESS(s2n_connection_set_config(server, forward_secret_config));
+
+            DEFER_CLEANUP(struct s2n_test_io_stuffer_pair test_io = { 0 },
+                    s2n_io_stuffer_pair_free);
+            EXPECT_OK(s2n_io_stuffer_pair_init(&test_io));
+            EXPECT_OK(s2n_connections_set_io_stuffer_pair(client, server, &test_io));
+
+            EXPECT_SUCCESS(s2n_negotiate_test_server_and_client(server, client));
+            EXPECT_EQUAL(client->actual_protocol_version, S2N_TLS13);
+            EXPECT_EQUAL(server->actual_protocol_version, S2N_TLS13);
+
+            uint16_t tickets_sent = 0;
+            EXPECT_SUCCESS(s2n_connection_get_tickets_sent(server, &tickets_sent));
+            EXPECT_EQUAL(tickets_sent, 1);
+        }
+
+        /* Server does not accept valid TLS1.2 ticket when forward secrecy is enforced */
+        {
+            DEFER_CLEANUP(struct s2n_connection *client = s2n_connection_new(S2N_CLIENT),
+                    s2n_connection_ptr_free);
+            EXPECT_NOT_NULL(client);
+            DEFER_CLEANUP(struct s2n_connection *server = s2n_connection_new(S2N_SERVER),
+                    s2n_connection_ptr_free);
+            EXPECT_NOT_NULL(server);
+
+            /* Disable forward secrecy for the first handshake */
+            EXPECT_SUCCESS(s2n_config_require_ticket_forward_secrecy(forward_secret_config, false));
+
+            EXPECT_SUCCESS(s2n_connection_set_config(client, tls12_client_config));
+            EXPECT_SUCCESS(s2n_connection_set_config(server, forward_secret_config));
+
+            DEFER_CLEANUP(struct s2n_test_io_stuffer_pair test_io = { 0 },
+                    s2n_io_stuffer_pair_free);
+            EXPECT_OK(s2n_io_stuffer_pair_init(&test_io));
+            EXPECT_OK(s2n_connections_set_io_stuffer_pair(client, server, &test_io));
+
+            EXPECT_SUCCESS(s2n_negotiate_test_server_and_client(server, client));
+            EXPECT_EQUAL(client->actual_protocol_version, S2N_TLS12);
+            EXPECT_EQUAL(server->actual_protocol_version, S2N_TLS12);
+
+            uint16_t tickets_sent = 0;
+            EXPECT_SUCCESS(s2n_connection_get_tickets_sent(server, &tickets_sent));
+            EXPECT_EQUAL(tickets_sent, 1);
+
+            uint8_t session_data[S2N_TLS12_SESSION_SIZE] = { 0 };
+            EXPECT_SUCCESS(s2n_connection_get_session(client, session_data, sizeof(session_data)));
+
+            /* Enable forward secrecy for the second handshake */
+            EXPECT_SUCCESS(s2n_config_require_ticket_forward_secrecy(forward_secret_config, true));
+
+            EXPECT_SUCCESS(s2n_connection_wipe(client));
+            EXPECT_SUCCESS(s2n_connection_wipe(server));
+
+            EXPECT_SUCCESS(s2n_connection_set_session(client, session_data, sizeof(session_data)));
+
+            EXPECT_SUCCESS(s2n_connection_set_config(client, tls12_client_config));
+            EXPECT_SUCCESS(s2n_connection_set_config(server, forward_secret_config));
+            EXPECT_OK(s2n_connections_set_io_stuffer_pair(client, server, &test_io));
+
+            EXPECT_SUCCESS(s2n_negotiate_test_server_and_client(server, client));
+            EXPECT_EQUAL(client->actual_protocol_version, S2N_TLS12);
+            EXPECT_EQUAL(server->actual_protocol_version, S2N_TLS12);
+
+            /* Session ticket not accepted */
+            EXPECT_TRUE(IS_FULL_HANDSHAKE(client));
+            EXPECT_TRUE(IS_FULL_HANDSHAKE(server));
+
+            /* No ticket issued */
+            EXPECT_SUCCESS(s2n_connection_get_tickets_sent(server, &tickets_sent));
+            EXPECT_EQUAL(tickets_sent, 0);
+        }
+    }
+
+    /* Test: Client disables tls12 tickets */
+    {
+        DEFER_CLEANUP(struct s2n_config *forward_secret_config = s2n_config_new(),
+                s2n_config_ptr_free);
+        EXPECT_NOT_NULL(forward_secret_config);
+        EXPECT_SUCCESS(s2n_config_require_ticket_forward_secrecy(forward_secret_config, true));
+        EXPECT_SUCCESS(s2n_config_set_cipher_preferences(forward_secret_config, "default_tls13"));
+        EXPECT_SUCCESS(s2n_config_set_session_tickets_onoff(forward_secret_config, 1));
+        EXPECT_SUCCESS(s2n_config_set_unsafe_for_testing(forward_secret_config));
+
+        DEFER_CLEANUP(struct s2n_config *tls12_server_config = s2n_config_new(),
+                s2n_config_ptr_free);
+        EXPECT_NOT_NULL(tls12_server_config);
+        EXPECT_SUCCESS(s2n_config_set_session_tickets_onoff(tls12_server_config, 1));
+        /* Security policy that does not support TLS1.3 */
+        EXPECT_SUCCESS(s2n_config_set_cipher_preferences(tls12_server_config, "20240501"));
+        EXPECT_SUCCESS(s2n_config_add_cert_chain_and_key_to_store(tls12_server_config,
+                chain_and_key));
+        EXPECT_SUCCESS(s2n_config_add_ticket_crypto_key(tls12_server_config, ticket_key_name1,
+                s2n_array_len(ticket_key_name1), ticket_key1, s2n_array_len(ticket_key1), 0));
+
+        DEFER_CLEANUP(struct s2n_config *tls13_server_config = s2n_config_new(),
+                s2n_config_ptr_free);
+        EXPECT_NOT_NULL(tls13_server_config);
+        EXPECT_SUCCESS(s2n_config_set_session_tickets_onoff(tls13_server_config, 1));
+        /* Security policy that does support TLS1.3 */
+        EXPECT_SUCCESS(s2n_config_set_cipher_preferences(tls13_server_config, "default_tls13"));
+        EXPECT_SUCCESS(s2n_config_add_cert_chain_and_key_to_store(tls13_server_config,
+                chain_and_key));
+        EXPECT_SUCCESS(s2n_config_add_ticket_crypto_key(tls13_server_config, ticket_key_name1,
+                s2n_array_len(ticket_key_name1), ticket_key1, s2n_array_len(ticket_key1), 0));
+
+        /* No ticket is received when forward secrecy is enforced and TLS1.2 is negotiated */
+        {
+            DEFER_CLEANUP(struct s2n_connection *client = s2n_connection_new(S2N_CLIENT),
+                    s2n_connection_ptr_free);
+            EXPECT_NOT_NULL(client);
+            DEFER_CLEANUP(struct s2n_connection *server = s2n_connection_new(S2N_SERVER),
+                    s2n_connection_ptr_free);
+            EXPECT_NOT_NULL(server);
+
+            EXPECT_SUCCESS(s2n_connection_set_config(client, forward_secret_config));
+            EXPECT_SUCCESS(s2n_connection_set_config(server, tls12_server_config));
+
+            DEFER_CLEANUP(struct s2n_test_io_stuffer_pair test_io = { 0 },
+                    s2n_io_stuffer_pair_free);
+            EXPECT_OK(s2n_io_stuffer_pair_init(&test_io));
+            EXPECT_OK(s2n_connections_set_io_stuffer_pair(client, server, &test_io));
+
+            EXPECT_SUCCESS(s2n_negotiate_test_server_and_client(server, client));
+
+            EXPECT_EQUAL(client->actual_protocol_version, S2N_TLS12);
+            EXPECT_EQUAL(server->actual_protocol_version, S2N_TLS12);
+
+            EXPECT_EQUAL(client->client_ticket.size, 0);
+        }
+
+        /* A ticket is received when forward secrecy is enforced and TLS1.3 is negotiated */
+        if (s2n_is_tls13_fully_supported()) {
+            DEFER_CLEANUP(struct s2n_connection *client = s2n_connection_new(S2N_CLIENT),
+                    s2n_connection_ptr_free);
+            EXPECT_NOT_NULL(client);
+            DEFER_CLEANUP(struct s2n_connection *server = s2n_connection_new(S2N_SERVER),
+                    s2n_connection_ptr_free);
+            EXPECT_NOT_NULL(server);
+
+            EXPECT_SUCCESS(s2n_connection_set_config(client, forward_secret_config));
+            EXPECT_SUCCESS(s2n_connection_set_config(server, tls13_server_config));
+
+            DEFER_CLEANUP(struct s2n_test_io_stuffer_pair test_io = { 0 },
+                    s2n_io_stuffer_pair_free);
+            EXPECT_OK(s2n_io_stuffer_pair_init(&test_io));
+            EXPECT_OK(s2n_connections_set_io_stuffer_pair(client, server, &test_io));
+
+            EXPECT_SUCCESS(s2n_negotiate_test_server_and_client(server, client));
+
+            EXPECT_EQUAL(client->actual_protocol_version, S2N_TLS13);
+            EXPECT_EQUAL(server->actual_protocol_version, S2N_TLS13);
+
+            /* Do a recv call to pick up TLS1.3 ticket */
+            uint8_t data = 1;
+            s2n_blocked_status blocked = S2N_NOT_BLOCKED;
+            EXPECT_FAILURE_WITH_ERRNO(s2n_recv(client, &data, 1, &blocked), S2N_ERR_IO_BLOCKED);
+
+            EXPECT_NOT_EQUAL(client->client_ticket.size, 0);
+        }
+    }
+
     EXPECT_SUCCESS(s2n_io_pair_close(&io_pair));
     EXPECT_SUCCESS(s2n_cert_chain_and_key_free(chain_and_key));
     EXPECT_SUCCESS(s2n_cert_chain_and_key_free(ecdsa_chain_and_key));
